package com.singularity.controller;

import cn.dev33.satoken.annotation.SaCheckRole;
import cn.hutool.core.util.ObjectUtil;
import com.singularity.aop.LogOperation;
import com.singularity.pojo.constant.AttachmentConstant;
import com.singularity.pojo.constant.FileBusinessConstant;
import com.singularity.pojo.dto.attachment.DeleteAttachmentListDTO;
import com.singularity.pojo.dto.attachment.QueryAttachmentListDTO;
import com.singularity.pojo.dto.attachment.UploadAttachmentDTO;
import com.singularity.pojo.dto.barking.*;
import com.singularity.common.entity.ChartDataModel;
import com.singularity.pojo.vo.barking.CompareBarkingDataVO;
import com.singularity.pojo.vo.barking.QueryBarkingDataFileListVO;
import com.singularity.pojo.vo.template.QueryAttachmentListVO;
import com.singularity.service.AttachmentService;
import com.singularity.service.BarkingChartDataService;
import com.singularity.util.FileUtil;
import com.singularity.util.TimedTaskUtil;
import com.singularity.util.result.CodeMessage;
import com.singularity.util.result.Result;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.io.IOException;
import java.util.List;
import java.util.UUID;

@Slf4j
@Api(tags = "行驶性能-制动性数据相关接口")
@RestController
@RequestMapping("api/barking")
public class BarkingController {

    @Resource
    private TimedTaskUtil timedTaskUtil;

    @Resource
    private BarkingChartDataService barkingChartDataService;

    @Resource
    private AttachmentService attachmentService;

    @Value("${file.path}")
    private String filePath;

    @ApiOperation("上传制动性试验数据文件")
    @LogOperation(message = "上传制动性试验数据文件")
    @PostMapping("/uploadFile")
    public Result<?> uploadBarkingFile(@Validated UploadBarkingDateFileDTO dto) {
        try {
            MultipartFile multipartFile = dto.getMultipartFile();
            dto.setOriginalFilename(multipartFile.getOriginalFilename());
            String uuid = UUID.randomUUID().toString().replace("-", "");
            String fileNameAndPath = filePath + uuid + multipartFile.getOriginalFilename();
            FileUtil.downloadToLocal(multipartFile.getInputStream(), fileNameAndPath);
            dto.setLocalFilePath(fileNameAndPath);
            dto.setSize(multipartFile.getSize());
        } catch (IOException e) {
            e.printStackTrace();
        }
        if (ObjectUtil.isNotEmpty(dto.getTaskStartTime())) {
            timedTaskUtil.startCron(
                    () -> barkingChartDataService.uploadBarkingFile(dto), dto.getTaskStartTime()
            );
        } else {
            barkingChartDataService.uploadBarkingFile(dto);
        }

        return Result.buildSuccessResult(CodeMessage.S_SUCCESS);
    }

    @ApiOperation("查询制动性试验数据文件列表")
    @PostMapping(value = "/queryBarkingFileList")
    public Result<List<QueryBarkingDataFileListVO>> queryBarkingDataFileList(@RequestBody @Validated QueryBarkingDataFileListDTO dto) {
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS, attachmentService.queryBarkingDataFileList(dto));
    }

    @SaCheckRole("admin")
    @ApiOperation("批量删除制动性试验数据文件")
    @LogOperation(message = "批量删除制动性试验数据文件")
    @PostMapping("/deleteFileList")
    public Result<?> deleteFileList(@Validated @RequestBody DeleteBarkingDataFileDTO dto) {
        barkingChartDataService.deleteFileList(dto);
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS);
    }

    @ApiOperation("查看制动性试验数据详情")
    @PostMapping("/queryBarkingData")
    public Result<ChartDataModel> queryBarkingData(@RequestBody @Validated QueryBarkingDataDTO dto) {
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS, barkingChartDataService.queryBarkingData(dto));
    }

    @ApiOperation("对比制动性试验数据结果")
    @LogOperation(message = "对比平顺性试验结果")
    @PostMapping("/compareRideData")
    public Result<CompareBarkingDataVO> compareBarkingData(@RequestBody @Validated CompareBarkingDataDTO dto) {

        return Result.buildSuccessResult(CodeMessage.S_SUCCESS, barkingChartDataService.compareBarkingData(dto));
    }

    @ApiOperation("行驶性能制动性试验报告主列表分页查询")
    @LogOperation(message = "行驶性能制动性试验报告主列表分页查询")
    @PostMapping(value = "/queryBarkingFileReportList")
    public Result<List<QueryAttachmentListVO>> queryBarkingFileReportList(@Validated @RequestBody QueryAttachmentListDTO dto) {
        dto.setBusinessId(FileBusinessConstant.BARKING_REPORT_FILE);
        dto.setOwnerType(AttachmentConstant.OWNER_TYPE_CAR);
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS, attachmentService.queryAttachmentList(dto));
    }

    @ApiOperation("上传行驶性能制动性试验报告文件")
    @LogOperation(message = "上传行驶性能制动性试验报告文件")
    @PostMapping(value = "/uploadBarkingFileReport")
    public Result<List<QueryAttachmentListVO>> uploadBarkingFileReport(@Validated UploadAttachmentDTO dto) {
        dto.setOwnerType(AttachmentConstant.OWNER_TYPE_CAR);
        dto.setBusinessType(FileBusinessConstant.BARKING_REPORT_FILE);
        attachmentService.uploadAttachment(dto);
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS);
    }

    @SaCheckRole("admin")
    @ApiOperation("删除行驶性能制动性试验报告文件")
    @LogOperation(message = "删除行驶性能制动性试验报告文件")
    @PostMapping(value = "/deleteBarkingFileReport")
    public Result<List<QueryAttachmentListVO>> deleteBarkingFileReport(@Validated @RequestBody DeleteAttachmentListDTO dto) {
        attachmentService.removeByIds(dto.getAttachIdList());
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS);
    }
}
