package com.singularity.controller;

import cn.dev33.satoken.annotation.SaCheckRole;
import cn.dev33.satoken.util.SaResult;
import cn.hutool.core.util.ObjectUtil;
import com.github.xiaoymin.knife4j.annotations.ApiOperationSupport;
import com.singularity.aop.LogOperation;
import com.singularity.pojo.dto.experiment.ExperimentFileDTO;

import com.singularity.pojo.dto.kc.*;
import com.singularity.pojo.entity.Attachment;
import com.singularity.pojo.dto.file.DeleteNFileDTO;
import com.singularity.pojo.dto.file.DownloadFileDTO;
import com.singularity.pojo.param.kc.*;
import com.singularity.pojo.vo.attachment.SelectAttachmentByFileNameVO;
import com.singularity.pojo.vo.car.*;
import com.singularity.service.AttachmentService;
import com.singularity.service.ConditionInfoService;
import com.singularity.service.KCService;
import com.singularity.pojo.vo.kc.*;
import com.singularity.exception.ToolsBusinessWarn;
import com.singularity.util.FileUtil;
import com.singularity.util.TimedTaskUtil;
import com.singularity.util.result.CodeMessage;
import com.singularity.util.result.Result;
import com.singularity.util.page.ResponseDTOWithPage;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.List;
import java.util.UUID;

@Slf4j
@RestController
@Api(tags = "KC相关接口")
@RequestMapping(value = "/api/kc")
public class KCController {

    @Resource
    private KCService kcService;

    @Resource
    private AttachmentService attachmentService;

    @Resource
    private TimedTaskUtil timedTaskUtil;

    @Resource
    private ConditionInfoService conditionInfoService;

    @Value("${file.path}")
    private String filePath;

    /******************************* K&C 原始数据相关接口************************/
    @ApiOperation("单个上传K&C试验数据文件")
    @LogOperation(message = "单个上传K&C试验数据文件")
    @PostMapping("/uploadKCDataFile")
    public Result<?> uploadKCFileData(@Validated UploadKCDataFile uploadKCDataFile) {
        kcService.uploadKCFileData(uploadKCDataFile);
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS);
    }

    @ApiOperationSupport(author = "Long XingYu")
    @ApiOperation("批量上传K&C试验数据文件")
    @LogOperation(message = "批量上传K&C试验数据文件")
    @PostMapping("/uploadKCConditions")
    public Result<?> uploadKCConditions(@Validated ExperimentFileDTO experimentFileDTO) {
        try {
            MultipartFile multipartFile = experimentFileDTO.getMultipartFile();
            experimentFileDTO.setOriginalFilename(multipartFile.getOriginalFilename());
            String uuid = UUID.randomUUID().toString().replace("-", "");
            String fileNameAndPath = filePath + uuid + multipartFile.getOriginalFilename();
            FileUtil.downloadToLocal(multipartFile.getInputStream(), fileNameAndPath);
            experimentFileDTO.setLocalFilePath(fileNameAndPath);
            experimentFileDTO.setSize(multipartFile.getSize());
        } catch (IOException e) {
            e.printStackTrace();
        }
        if(ObjectUtil.isNotEmpty(experimentFileDTO.getTaskStartTime())){
            timedTaskUtil.startCron(
                    () -> kcService.asyncUploadKCConditions(experimentFileDTO),experimentFileDTO.getTaskStartTime()
            );
        }else{
            kcService.asyncUploadKCConditions(experimentFileDTO);
        }
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS);
    }

    @ApiOperation("修改试验方案")
    @LogOperation(message = "K&C修改试验方案")
    @PostMapping(value = "/insertExperimentCondition")
    public Result<Integer> insertExperimentCondition(@RequestBody @Validated SetExperimentParamDTO setExperimentParamDTO) {
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS,
                kcService.insertExperimentCondition(setExperimentParamDTO));
    }

    @SaCheckRole("admin")
    @ApiOperation("批量删除KC附件")
    @LogOperation(message = "删除K&C试验数据文件")
    @PostMapping("/deleteFile")
    public Result<Integer> deleteAttachmentByIdList(@RequestBody @Validated DeleteNFileDTO deleteNFileDTO) {
        kcService.deleteKCAttachmentByIdList(deleteNFileDTO);
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS);
    }

    @ApiOperation("KC附件下载")
    @PostMapping("/downloadFile")
    public Result<String> downloadFile(@RequestBody @Validated DownloadFileDTO downloadFileDTO) {
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS,
                attachmentService.getAttachmentById(downloadFileDTO.getId()).getPath());
    }

    @ApiOperation("查询KC试验报告文件列表")
    @PostMapping("/selectFileByName")
    public Result<ResponseDTOWithPage<SelectAttachmentByFileNameVO>> selectFileByName(@RequestBody @Validated KCFileSearch kcFileSearch) {
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS,
                attachmentService.selectByFileMessages(kcFileSearch));
    }

    @ApiOperation("根据excel导入数据文件")
    @PostMapping("/importFileByName")
    @LogOperation(message = "车辆信息导入")
    public Result<?> importFileByName(KCUploadCarFileDTO kcUploadCarFileDTO) {
        if (kcUploadCarFileDTO.getMultipartFile() == null)
            throw new ToolsBusinessWarn("文件列表不能为空");
        kcService.importFileByName(kcUploadCarFileDTO);
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS);
    }

    @ApiOperationSupport(author = "wanxing")
    @ApiOperation("获取K&C关键指标信息")
    @PostMapping("/queryKCEvaluationInfo")
    public Result<List<QueryKCEvaluationInfoVO>> queryKCEvaluationInfo(@RequestBody @Validated QueryKCEvaluationInfoDTO dto) {
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS,kcService.queryKCEvaluationInfo(dto));
    }

    @ApiOperationSupport(author = "wanxing")
    @ApiOperation("对比K&C关键指标信息")
    @PostMapping("/compareKCEvaluationInfo")
    public Result<CompareKCEvaluationInfoVO> compareKCEvaluationInfo(@RequestBody @Validated CompareKCEvaluationInfoDTO dto) {
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS,kcService.compareKCEvaluationInfo(dto));
    }

    @ApiOperation("上传K&C试验报告文件")
    @LogOperation(message = "上传K&C试验报告文件")
    @PostMapping("/uploadKCReportFile")
    public Result<List<Attachment>> uploadKCReportFile(@Validated UploadKCReportFileDTO dto) {
        if(dto.getMultipartFile().getOriginalFilename().contains(".zip")){
            attachmentService.uploadZipAttachment(
                    dto.getMultipartFile(),
                    dto.getOwnerType(),
                    dto.getOwnerId(),
                    dto.getBusinessType(),
                    dto.getOperId()
            );
        }else{
            attachmentService.uploadAttachment(
                    dto.getMultipartFile(),
                    dto.getOwnerType(),
                    dto.getOwnerId(),
                    dto.getBusinessType(),
                    dto.getOperId()
            );
        }
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS);
    }

    @ApiOperation("导出KC关键指标数据")
    @PostMapping("/exportKCEvaluationInfo")
    public void exportKCEvaluationInfo(@RequestBody @Validated ExportKCEvaluationInfoDTO dto, HttpServletResponse response) {
        kcService.exportKCEvaluationInfo(dto, response);
    }

    @ApiOperation("查询K&C工况测点信息")
    @PostMapping("/queryKCConditionInfo")
    public Result<List<QueryKCConditionInfoVO>> queryKCConditionInfo() {
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS, conditionInfoService.queryKCConditionInfo());
    }

    @ApiOperation("批量查询K&C数据文件列表")
    @PostMapping("/queryKCDataFileList")
    public Result<QueryKCAttachmentListVO> queryKCDataFileList(@Validated @RequestBody QueryKCAttachmentListDTO dto) {
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS, conditionInfoService.queryKCDataFileList(dto));
    }
    /******************************* K&C 关键指标相关接口************************/
    @ApiOperation("获取实验条件及工况")
    @PostMapping(value = "/getExperimentCondition")
    public Result<ExperimentConditionDTO> getExperimentCondition(@RequestBody @Validated ExperimentConditionParam experimentConditionParam) {
        ExperimentConditionDTO experimentConditionDTO = kcService.getExperimentCondition(experimentConditionParam);
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS, experimentConditionDTO);
    }

    @ApiOperation("获取关键指标列表")
    @PostMapping(value = "/getKeyIndexList")
    public Result<ResponseDTOWithPage<KeyIndexDTO>> getKeyIndexList(@RequestBody @Validated KeyIndexParam keyIndexParam) {
        ResponseDTOWithPage<KeyIndexDTO> keyIndexDTOS = kcService.getKeyIndexList(keyIndexParam);
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS, keyIndexDTOS);
    }

    /******************************* K&C 试验曲线相关接口************************/
    @ApiOperation("读取数据库K&C数据文件")
    @PostMapping(value = "/readKCFile")
    public Result<List<ReadKCFileVO>> readKCFile(@RequestBody @Validated ReadKCFileDTO dto) {
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS, kcService.readKCFile(dto));
    }

    @ApiOperation("批量读取数据库K&C数据文件")
    @PostMapping(value = "/readKCFileList")
    public Result<List<ReadKCFileListVO>> readKCFileList(@RequestBody @Validated ReadKCFileListDTO dto) {
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS, kcService.readKCFileList(dto));
    }

    /****************************** 车辆信息相关接口***************************/
    @ApiOperation("车辆信息主列表分页查询")
    @PostMapping(value = "/getCarDataList")
    public Result<ResponseDTOWithPage<CarDataDTO>> getCarDataList(@Validated @RequestBody CarDataParam carDataParam) {
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS, kcService.getCarDataList(carDataParam));
    }

    @ApiOperation("添加车辆数据列表")
    @LogOperation(message = "添加车辆数据列表")
    @PostMapping(value = "/addCarDataList")
    public Result<ResponseDTOWithPage<CarDataDTO>> addCarDataList(@Validated @RequestBody CarDataAddParam carDataAddParam) {
        return kcService.addCarDataList(carDataAddParam) > 0 ? Result.buildSuccessResult(CodeMessage.S_SUCCESS) : Result.buildFailureResult(CodeMessage.E_ERROR);
    }

    @ApiOperation("获取车辆数据统计")
    @PostMapping(value = "/getCarDataStatistics")
    public SaResult getCarDataStatistics(@RequestBody CarDataStatisticsParam carDataStatisticsParam) {
        CarDataStatisticsDTO carDataStatisticsDTO = kcService.getCarDataStatistics(carDataStatisticsParam);
        return new SaResult(SaResult.CODE_SUCCESS, "操作成功", carDataStatisticsDTO);
    }

    @ApiOperation("查询车辆基本参数")
    @PostMapping(value = "/selectCarBasicParam")
    public Result<List<CarBasicParamVO>> selectCarBasicParam(@RequestBody @Validated CarMessageIdDTO carMessageIdDTO) {
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS,
                kcService.selectCarBasicParam(carMessageIdDTO));
    }

    @ApiOperation("查询车辆车身参数")
    @PostMapping(value = "/selectCarBodyParam")
    public Result<List<CarBodyParamVO>> selectCarBodyParam(@RequestBody @Validated CarMessageIdDTO carMessageIdDTO) {
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS,
                kcService.selectCarBodyParam(carMessageIdDTO));
    }

    @ApiOperation("查询车辆底盘结构")
    @PostMapping(value = "/selectCarChassisStructure")
    public Result<List<CarChassisStructureVO>> selectCarChassisStructure(@RequestBody @Validated CarMessageIdDTO carMessageIdDTO) {
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS,
                kcService.selectCarChassisStructure(carMessageIdDTO));
    }

    @ApiOperation("查询车辆动力参数")
    @PostMapping(value = "/selectCarEnergyParam")
    public Result<List<SelectCarEnergyParamVO>> selectCarEnergyParam(@RequestBody @Validated CarMessageIdDTO carMessageIdDTO) {
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS,
                kcService.selectCarEnergyParam(carMessageIdDTO));
    }

    @ApiOperation("查询车辆试验参数列表")
    @PostMapping(value = "/queryCarExperimentParamList")
    public Result<List<CarExperimentParamListQueryVO>> queryCarExperimentParamList(@RequestBody @Validated CarExperimentParamListQueryDTO dto) {
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS,
                attachmentService.queryCarExperimentParamList(dto));
    }

    @ApiOperation("修改车辆基本参数")
    @LogOperation(message = "修改车辆基本参数")
    @PostMapping("/updateCarBasicParam")
    public Result<?> updateCarBasicParam(@RequestBody @Validated CarFileDTO carMessageIdDTO) {
        kcService.updateCarBasicParam(carMessageIdDTO);
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS);
    }

    @ApiOperation("修改车辆车身参数")
    @LogOperation(message = "修改车辆车身参数")
    @PostMapping("/updateCarBodyParam")
    public Result<?> updateCarBodyParam(@RequestBody @Validated CarFileDTO carMessageIdDTO) {
        kcService.updateCarBodyParam(carMessageIdDTO);
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS);
    }

    @ApiOperation("修改车辆底盘结构")
    @LogOperation(message = "修改车辆底盘结构")
    @PostMapping("/updateCarChassisStructure")
    public Result<?> updateCarChassisStructure(@RequestBody @Validated CarFileDTO carMessageIdDTO) {
        kcService.updateCarChassisStructure(carMessageIdDTO);
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS);
    }

    @ApiOperation("修改车辆动力参数")
    @LogOperation(message = "修改车辆动力参数")
    @PostMapping("/updateCarEnergyParam")
    public Result<?> updateCarEnergyParam(@RequestBody @Validated CarFileDTO carMessageIdDTO) {
        kcService.updateCarEnergyParam(carMessageIdDTO);
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS);
    }
}
