package com.singularity.controller;

import cn.dev33.satoken.annotation.SaCheckRole;
import cn.dev33.satoken.util.SaResult;
import cn.hutool.core.util.ObjectUtil;
import com.singularity.aop.LogOperation;
import com.singularity.common.vo.nvh.ReadNVHFileVO;
import com.singularity.pojo.dto.nvh.*;
import com.singularity.pojo.param.nvh.NVHContentAddParam;
import com.singularity.pojo.param.nvh.NVHSelectParam;
import com.singularity.pojo.param.nvh.NVHUpdateParam;
import com.singularity.pojo.param.nvh.NVHUploadFileParam;
import com.singularity.service.NVHService;
import com.singularity.pojo.vo.nvh.*;
import com.singularity.util.FileUtil;
import com.singularity.util.TimedTaskUtil;
import com.singularity.util.result.CodeMessage;
import com.singularity.util.result.Result;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.io.IOException;
import java.util.List;
import java.util.UUID;

@Slf4j
@Api(tags = "NVH相关接口")
@RestController
@RequestMapping("api/nvh")
public class NVHController {

    @Resource
    private NVHService nvhService;

    @Resource
    private TimedTaskUtil timedTaskUtil;

    @Value("${file.path}")
    private String filePath;

    @ApiOperation("NVH文件上传")
    @LogOperation(message = "上传NVH数据文件")
    @PostMapping("/uploadFile")
    public SaResult uploadFile(NVHUploadFileParam fileParam) {
        nvhService.uploadNVHFile(fileParam);
        return SaResult.ok();
    }

    @ApiOperation(value = "NVH文件批量上传",notes = "上传文件格式为zip压缩包")
    @LogOperation(message = "批量上传NVH数据文件")
    @PostMapping("/uploadFileList")
    public SaResult uploadFileList(UploadFileListDTO dto){
        try {
            MultipartFile multipartFile = dto.getMultipartFile();
            dto.setOriginalFilename(multipartFile.getOriginalFilename());
            String uuid = UUID.randomUUID().toString().replace("-", "");
            String fileNameAndPath = filePath + uuid + multipartFile.getOriginalFilename();
            FileUtil.downloadToLocal(multipartFile.getInputStream(), fileNameAndPath);
            dto.setLocalFilePath(fileNameAndPath);
            dto.setSize(multipartFile.getSize());
        } catch (IOException e) {
            e.printStackTrace();
        }
        if(ObjectUtil.isNotEmpty(dto.getTaskStartTime())){
            timedTaskUtil.startCron(
                    () -> nvhService.asyncUploadFileList(dto),dto.getTaskStartTime()
            );
        }else{
            nvhService.asyncUploadFileList(dto);
        }
        return SaResult.ok();
    }

    @ApiOperation("读取NVH数据文件")
    @PostMapping("/readNVHFile")
    public Result<List<ReadNVHFileVO>> readNVHFile(@Validated @RequestBody ReadNVHFileDTO dto) {
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS,nvhService.readNVHFile(dto));
    }

    @ApiOperation("批量读取NVH数据文件")
    @PostMapping("/readNVHFileList")
    public Result<ReadNVHFileListVO> readNVHFileList(@Validated @RequestBody ReadNVHFileListDTO dto) {
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS,nvhService.readNVHFileList(dto));
    }

    @ApiOperation("查询NVH菜单树列表")
    @PostMapping("/getNVHMeuTreeList")
    public Result<List<GetNVHMenuTreeVO>> getNVHMeuTreeList(@RequestBody GetNVHMenuTreeDTO dto) {
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS,nvhService.getNVHMeuTreeList(dto));
    }

    @ApiOperation("添加NVH目录列表")
    @PostMapping("/addNVHContentList")
    public SaResult addNVHContentList(@RequestBody NVHContentAddParam addParam) {
        return nvhService.addNVHContentList(addParam) > 0 ? SaResult.ok("添加成功") : SaResult.error("添加失败");
    }

    @ApiOperation("NVH对比菜单树列表")
    @PostMapping("/getNVHContentListForCompare")
    public SaResult getNVHContentListForCompare(@RequestBody NVHSelectParam selectParam) {
        return new SaResult(SaResult.CODE_SUCCESS, "获取成功", nvhService.getNVHContentListForCompare(selectParam));
    }

    @SaCheckRole("admin")
    @LogOperation(message = "删除NVH目录")
    @DeleteMapping("/deleteNVHContentList/{id}")
    public SaResult deleteNVHContentList(@PathVariable("id") Long id) {
        nvhService.deleteNVHContentList(id);
        return SaResult.ok("删除成功");
    }

    @ApiOperation("NVH目录修改")
    @PostMapping("/updateNVHContentList")
    public SaResult updateNVHContentList(@RequestBody NVHUpdateParam updateParam) {
        return nvhService.updateNVHContentList(updateParam) > 0 ? SaResult.ok("修改成功") : SaResult.error("修改失败");
    }

    @SaCheckRole("admin")
    @ApiOperation("NVH文件删除")
    @LogOperation(message = "NVH文件删除")
    @DeleteMapping("/deleteNVHFile/{id}")
    public SaResult deleteNVHFile(@PathVariable("id") Long id) {
        return nvhService.deleteNVHFile(id) > 0 ? SaResult.ok("删除成功") : SaResult.error("删除失败");
    }

    @ApiOperation("对比NVH性能主观评价数据文件")
    @PostMapping("/compareNVHSubjectiveFile")
    public Result<CompareNVHSubjectiveFileVO> compareNVHSubjectiveFile(@Validated @RequestBody CompareNVHSubjectiveFileDTO dto) {
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS,nvhService.compareNVHSubjectiveFile(dto));
    }

    @ApiOperation("对比NVH性能客观评价数据文件")
    @PostMapping("/compareNVHObjectiveFile")
    public Result<CompareNVHObjectiveFileVO> compareNVHObjectiveFile(@Validated @RequestBody CompareNVHObjectiveFileDTO dto) {
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS,nvhService.compareNVHObjectiveFile(dto));
    }

    @ApiOperation("批量查询NVH客观评价数据文件列表")
    @PostMapping("/queryNVHDataFileList")
    public Result<QueryNVHDataFileListVO> queryNVHDataFileList(@Validated @RequestBody QueryNVHDataFileListDTO dto) {
        return Result.buildSuccessResult(CodeMessage.S_SUCCESS, nvhService.queryNVHDataFileList(dto));
    }


}
