package com.singularity.listener;

import com.alibaba.excel.context.AnalysisContext;
import com.alibaba.excel.read.listener.ReadListener;
import com.singularity.common.entity.ChartDataModel;
import com.singularity.common.entity.PartChartData;
import com.singularity.pojo.entity.DamperData;
import com.singularity.service.DamperChartDataService;
import com.singularity.service.DamperDataService;
import com.singularity.util.FileUtil;
import lombok.extern.slf4j.Slf4j;

import java.io.IOException;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

@Slf4j
public class DamperChartDataListener implements ReadListener<Map<Integer, String>> {

    private final DamperChartDataService damperChartDataService;

    private final DamperDataService damperDataService;

    // 减振器类型
    private final Integer damperType;

    // 附件id
    private final Long attachId;

    // 车辆id
    private final Long carId;

    // 当前时间
    LocalDateTime localDateTime = LocalDateTime.now();

    // 减振器数据集合
    List<DamperData> damperDataList = new ArrayList<>();

    public DamperChartDataListener(DamperChartDataService damperChartDataService, Long attachId,Long carId,
                                   Integer damperType,DamperDataService damperDataService

    ) {
        this.damperDataService = damperDataService;
        this.damperChartDataService = damperChartDataService;
        this.attachId = attachId;
        this.carId = carId;
        this.damperType = damperType;

    }

    @Override
    public void invoke(Map<Integer, String> data, AnalysisContext context) {
        DamperData damperData = new DamperData();
        damperData.setCarId(carId);
        damperData.setAttachId(attachId);
        damperData.setDamperType(damperType);
        damperData.setFrequency(data.get(0) != null ? Double.parseDouble(data.get(0)) : null);
        damperData.setAmplitude(data.get(1) != null ? Double.parseDouble(data.get(1)) : null);
        damperData.setReboundVel(data.get(2) != null ? Double.parseDouble(data.get(2)) : null);
        damperData.setRebound(data.get(3) != null ? Double.parseDouble(data.get(3)) : null);
        damperData.setCompressionVel(data.get(4) != null ? Double.parseDouble(data.get(4)) : null);
        damperData.setCompression(data.get(5) != null ? Double.parseDouble(data.get(5)) : null);
        damperData.setCreateTime(localDateTime);
        damperData.setUpdateTime(localDateTime);
        damperDataList.add(damperData);
    }

    @Override
    public void doAfterAllAnalysed(AnalysisContext context) {
        // 保存减振器数据
        damperDataService.saveBatch(damperDataList);

        // 保存减振器绘图数据
        this.saveDamperChartData();
    }

    /**
     * 保存图像数据
     */
    private void saveDamperChartData(){
        String title = damperType == 0 ? "前减振器位移/力值对比图" : "后减振器位移/力值对比图";
        ChartDataModel chartDataModel = new ChartDataModel();
        chartDataModel.setXLabel("Peak Velocity (m/s)");
        chartDataModel.setYLabel("Compression < Peak Force (N)>Rebound");
        List<String> legendList = new ArrayList<>();
        legendList.add("复原");
        legendList.add("压缩");
        chartDataModel.setLegendList(legendList);
        chartDataModel.setLegend("");
        chartDataModel.setTitle(title);
        double[][] data = new double[4][];
        double[] dataX = new double[damperDataList.size()];
        double[] dataY1 = new double[damperDataList.size()];
        double[] dataY2 = new double[damperDataList.size()];
        for (int i = 0; i < damperDataList.size(); i++) {
            DamperData damperData = damperDataList.get(i);
            dataX[i] = damperData.getReboundVel();
            dataY1[i] = damperData.getRebound();
            dataY2[i] = damperData.getCompression();
        }
        data[0] = dataX;
        data[1] = dataY1;
        data[2] = dataX;
        data[3] = dataY2;
        chartDataModel.setData(data);
        PartChartData partChartData = new PartChartData();
        partChartData.setCreateTime(localDateTime);
        partChartData.setUpdateTime(localDateTime);
        partChartData.setAttachId(attachId);
        try {
            partChartData.setChartData(FileUtil.objectToByte(chartDataModel));
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
        damperChartDataService.save(partChartData);
    }
}
