package com.singularity.service.impl;

import cn.hutool.core.util.StrUtil;
import com.singularity.pojo.entity.Attachment;
import com.singularity.pojo.vo.attachment.UploadAttachmentVO;
import com.singularity.service.MinioService;
import io.minio.*;
import io.minio.http.Method;
import io.minio.messages.DeleteError;
import io.minio.messages.DeleteObject;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;
import java.util.UUID;

@Slf4j
@Service
public class MinioServiceImpl implements MinioService {

    @Resource
    private MinioClient minioClient;

    @Value("${detection.minio.bucketName}")
    private String bucketName;

    @Value("${detection.minio.pathPrefix}")
    private String pathPrefix;

    @Value("${detection.minio.endpoint}")
    private String endpoint;

    @Value("${detection.minio.virtualUrl}")
    private String virtualUrl;

    @Override
    @SneakyThrows
    public UploadAttachmentVO storeFile(MultipartFile file) {
        UploadAttachmentVO uploadAttachmentVO = new UploadAttachmentVO();
        String uuid = UUID.randomUUID().toString().replace("-", "");
        String objectName = pathPrefix + StrUtil.SLASH + uuid + "-" + file.getOriginalFilename();
        InputStream inputStream = file.getInputStream();
        minioClient.putObject(
                PutObjectArgs.builder()
                        .bucket(bucketName)
                        .object(objectName)
                        .stream(inputStream, file.getSize(), -1)
                        .contentType(file.getContentType())
                        .build()
        );
        uploadAttachmentVO.setName(file.getOriginalFilename());
        uploadAttachmentVO.setObjName(objectName);
        String fileUrl = this.getFileUrl(objectName, null);
        uploadAttachmentVO.setUrl(fileUrl);
        uploadAttachmentVO.setPath(fileUrl.substring(0, fileUrl.indexOf("?")));
        String[] split = Objects.requireNonNull(file.getOriginalFilename()).split("\\.");
        uploadAttachmentVO.setType(split[split.length-1]);
        uploadAttachmentVO.setSize(file.getSize());
        inputStream.close();
        return uploadAttachmentVO;
    }

    @Override
    @SneakyThrows
    public Attachment uploadFile(MultipartFile file){
        Attachment attachment = new Attachment();
        String uuid = UUID.randomUUID().toString().replace("-", "");
        String objectName = pathPrefix + StrUtil.SLASH + uuid + "-" + file.getOriginalFilename();
        InputStream inputStream = file.getInputStream();
        minioClient.putObject(
                PutObjectArgs.builder()
                        .bucket(bucketName)
                        .object(objectName)
                        .stream(inputStream, file.getSize(), -1)
                        .contentType(file.getContentType())
                        .build()
        );
        attachment.setName(file.getOriginalFilename());
        attachment.setObjName(objectName);
        String fileUrl = this.getFileUrl(objectName, null);
        attachment.setPath(fileUrl.substring(0, fileUrl.indexOf("?")));
        String[] split = Objects.requireNonNull(file.getOriginalFilename()).split("\\.");
        attachment.setType(split[split.length-1]);
        attachment.setSize(file.getSize());
        inputStream.close();
        return attachment;
    }

    @Override
    public List<Attachment> storeFiles(List<MultipartFile> multipartFileList) {
        List<Attachment> attachmentList = new ArrayList<>();
        for (MultipartFile multipartFile : multipartFileList) {
            attachmentList.add(this.uploadFile(multipartFile));
        }
        return attachmentList;
    }

    /**
     * 文件上传
     */
    @Override
    @SneakyThrows
    public UploadAttachmentVO storeFile(InputStream in, String fileName,long size) {
        UploadAttachmentVO uploadAttachmentVO = new UploadAttachmentVO();
        String uuid = UUID.randomUUID().toString().replace("-", "");
        String objectName = pathPrefix + StrUtil.SLASH + uuid + "-" + fileName;
        minioClient.putObject(
                PutObjectArgs.builder()
                        .bucket(bucketName)
                        .object(objectName)
                        .stream(in,size, -1)
                        .build()
        );
        uploadAttachmentVO.setName(fileName);
        uploadAttachmentVO.setObjName(objectName);
        String fileUrl = this.getFileUrl(objectName, null);
        uploadAttachmentVO.setUrl(fileUrl);
        uploadAttachmentVO.setPath(fileUrl.substring(0, fileUrl.indexOf("?")));
        uploadAttachmentVO.setType(Objects.requireNonNull(fileName).split("\\.")[1]);
        uploadAttachmentVO.setSize(size);
        return uploadAttachmentVO;
    }

    /**
     * 文件下载
     */
    @Override
    @SneakyThrows
    public InputStream getFile(String objectName) {
        return minioClient.getObject(
                GetObjectArgs.builder()
                        .bucket(bucketName)
                        .object(objectName)
                        .build());
    }

    /**
     * 预览图片(默认时间为一小时)
     */
    @Override
    @SneakyThrows
    public String getFileUrl(String objectName, Integer expiryTime) {
        if (Objects.isNull(expiryTime)) {
            expiryTime = 60 * 60;
        }
        // 查看文件地址
        String url = minioClient.getPresignedObjectUrl(
                GetPresignedObjectUrlArgs
                        .builder()
                        .bucket(bucketName)
                        .object(objectName)
                        .method(Method.GET)
                        .expiry(expiryTime)
                        .build());
        //用虚拟地址替换实际地址
        url = url.replace(endpoint, virtualUrl);
        return url;
    }

    /**
     * 删除
     */
    @Override
    @SneakyThrows
    public void remove(String objectName) {
        minioClient.removeObject(
                RemoveObjectArgs
                        .builder()
                        .bucket(bucketName)
                        .object(objectName)
                        .build());
    }

    /**
     * 批量删除
     */
    @Override
    @SneakyThrows
    public void removeList(List<String> objectNames) {
        List<DeleteObject> deleteObjectList = new ArrayList<>();
        objectNames.forEach(objectName -> {
            deleteObjectList.add(new DeleteObject(objectName));
        });

        RemoveObjectsArgs removeObjectsArgs =
                RemoveObjectsArgs
                        .builder()
                        .bucket(bucketName)
                        .objects(deleteObjectList)
                        .build();
        for (Result<DeleteError> errorResult : minioClient.removeObjects(removeObjectsArgs)) {
            DeleteError error = errorResult.get();
            log.error("删除失败 '" + error.objectName() + "'. 错误原因:" + error.message());
        }
    }
}
