package com.singularity.service.impl;

import cn.dev33.satoken.exception.SaTokenException;
import cn.dev33.satoken.secure.BCrypt;
import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollectionUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.singularity.pojo.param.user.*;
import com.singularity.dao.UserMapper;
import com.singularity.dao.UserRoleMapper;
import com.singularity.pojo.dto.role.RoleInfoDTO;
import com.singularity.pojo.dto.user.UserDetailDTO;
import com.singularity.pojo.entity.User;
import com.singularity.pojo.entity.UserRole;
import com.singularity.service.UserService;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.singularity.pojo.dto.user.UserListDTO;
import com.singularity.service.RoleService;
import com.singularity.util.page.PageUtil;
import com.singularity.util.page.ResponseDTOWithPage;
import org.springframework.transaction.annotation.Transactional;


import java.util.*;
import java.util.stream.Collectors;

@Service
public class UserServiceImpl implements UserService {

    private final UserMapper userMapper;

    private final RoleService roleService;

    private final UserRoleMapper userRoleMapper;

    @Autowired
    public UserServiceImpl(UserMapper userMapper, RoleService roleService, UserRoleMapper userRoleMapper) {
        this.userMapper = userMapper;
        this.roleService = roleService;
        this.userRoleMapper = userRoleMapper;
    }

    @Override
    public ResponseDTOWithPage<UserListDTO> getUserList(UserListParam userListParam) {
        PageUtil.startPage(userListParam, true, UserListDTO.class);
        List<UserListDTO> list = userMapper.getUserList(userListParam);

        List<Long> userIdList = list.stream().map(UserListDTO::getId).collect(Collectors.toList());
        LambdaQueryWrapper<UserRole> queryWrapper = Wrappers.lambdaQuery();
        queryWrapper.select(UserRole::getUserId,UserRole::getRoleId)
                .in(CollectionUtil.isNotEmpty(list),UserRole::getUserId,userIdList);
        List<UserRole> roleIdList = userRoleMapper.selectList(queryWrapper);
        Map<Long,List<UserRole>> map = roleIdList.stream().collect(Collectors.groupingBy(UserRole::getUserId));
        for (UserListDTO userListDTO : list) {
            if(map.containsKey(userListDTO.getId())){
                List<Long> roleList = map.get(userListDTO.getId()).stream().map(UserRole::getRoleId).collect(Collectors.toList());
                userListDTO.setRoleList(roleList);
            }
        }
        return PageUtil.getListDTO(list,userListParam);
    }


    @Override
    public User getUserByPhone(Long phone) {
        LambdaQueryWrapper<User> userLambdaQueryWrapper = new LambdaQueryWrapper<>();
        userLambdaQueryWrapper.eq(User::getPhone,phone).isNull(User::getDeletedAt);
        return userMapper.selectOne(userLambdaQueryWrapper);
    }

    @Override
    @Transactional
    public void delUserList(UserDelListParam userDelListParam) {
        // 删除用户信息
        LambdaQueryWrapper<User> wrapper = new LambdaQueryWrapper<>();
        wrapper.in(User::getId,userDelListParam.getIdList());
        userMapper.delete(wrapper);

        // 删除用户角色信息
        LambdaQueryWrapper<UserRole> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.in(UserRole::getUserId,userDelListParam.getIdList());
        userRoleMapper.delete(queryWrapper);
    }

    @Transactional
    @Override
    public void insert(UserAddParam userAddParam) {
        // 手机号验重
        Long phone = userAddParam.getPhone();
        User user = this.getUserByPhone(phone);
        if(!Objects.isNull(user))
            throw new SaTokenException(400, "该手机号已被注册");

        // 添加用户基本信息
        User user2 = new User();
        BeanUtil.copyProperties(userAddParam,user2);
        userMapper.insert(user2);

        // 添加用户角色关联关系
        // 修改用户角色信息
        if(CollectionUtil.isNotEmpty(userAddParam.getRoleList())){
            List<UserRole> userRoleList = new ArrayList<>();
            // 新建用户角色信息
            for (Long roleId : userAddParam.getRoleList()) {
                UserRole userRole = new UserRole();
                userRole.setUserId((long)user2.getId());
                userRole.setRoleId(roleId);
                userRoleList.add(userRole);
            }
            userRoleMapper.insertBatch(userRoleList);
        }
    }

    @Override
    public Boolean update(User user) {
        LambdaQueryWrapper<User> lqw = new LambdaQueryWrapper<>();
        lqw.eq(StringUtils.isNotEmpty(user.getPhone()),User::getPhone,user.getPhone()).ne(User::getId,user.getId());
        if(userMapper.selectCount(lqw) > 0)
            throw new SaTokenException("手机号已存在");
        lqw.eq(StringUtils.isNotEmpty(user.getNickname()),User::getNickname,user.getNickname()).ne(User::getId,user.getId());
        if(userMapper.selectCount(lqw) > 0)
            throw new SaTokenException("昵称已存在");
        return userMapper.updateById(user) > 0;
    }

    @Override
    public void update(UserUpdateParam userUpdateParam) {
        // 校验手机号、昵称、邮箱
        LambdaQueryWrapper<User> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(StringUtils.isNotEmpty(userUpdateParam.getPhone()), User::getPhone,userUpdateParam.getPhone())
                .ne(User::getId,userUpdateParam.getId());
        if(userMapper.selectCount(queryWrapper) > 0)
            throw new SaTokenException("手机号已存在");

        LambdaQueryWrapper<User> queryWrapper2 = new LambdaQueryWrapper<>();
        queryWrapper2.eq(StringUtils.isNotEmpty(userUpdateParam.getNickname()),User::getNickname,userUpdateParam.getNickname())
                .ne(User::getId,userUpdateParam.getId());
        if(userMapper.selectCount(queryWrapper2) > 0)
            throw new SaTokenException("昵称已存在");

        // 修改用户信息
        User user = new User();
        BeanUtil.copyProperties(userUpdateParam, user);
        userMapper.updateById(user);

        // 修改用户角色信息
        if(CollectionUtil.isNotEmpty(userUpdateParam.getRoleList())){
            // 删除用户角色信息
            LambdaQueryWrapper<UserRole> queryWrapper4 = new LambdaQueryWrapper<>();
            queryWrapper4.eq(UserRole::getUserId,userUpdateParam.getId());
            userRoleMapper.delete(queryWrapper4);

            List<UserRole> userRoleList = new ArrayList<>();
            // 新建用户角色信息
            for (Long roleId : userUpdateParam.getRoleList()) {
                UserRole userRole = new UserRole();
                userRole.setUserId(userUpdateParam.getId());
                userRole.setRoleId(roleId);
                userRoleList.add(userRole);
            }
            userRoleMapper.insertBatch(userRoleList);
        }
    }

    @Override
        public UserDetailDTO getUserInfo(Integer id) {
        UserDetailDTO userInfo = userMapper.getUserInfo(id);
        if(userInfo == null){
           throw new SaTokenException("用户不存在");
        }
        List<RoleInfoDTO> roleInfo = roleService.getRoleInfo(id);
        userInfo.setRoles(roleInfo);
        return userInfo;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Boolean distributeRole(DistributeRoleParam distributeRoleParam) {
        // 将原有角色删除
        LambdaUpdateWrapper<UserRole> updateWrapper = new LambdaUpdateWrapper<>();
        updateWrapper.eq(UserRole::getUserId,distributeRoleParam.getUserId());
        userRoleMapper.update(UserRole.builder().deletedAt(new Date()).build(), updateWrapper);
        // 添加新角色
        List<UserRole> userRoles = distributeRoleParam.getRoleIds().stream()
                .map(roleId -> UserRole.builder().userId(distributeRoleParam.getUserId()).roleId(roleId).build())
                .collect(Collectors.toList());
        int insertBatch = userRoleMapper.insertBatch(userRoles);
        return insertBatch == userRoles.size();
    }

    @Override
    public Boolean updatePsw(UpdatePswParam updatePswParam) {
        User user = userMapper.selectById(updatePswParam.getId());
        if(user == null){
            throw new SaTokenException("用户不存在");
        }
        if(!BCrypt.checkpw(updatePswParam.getOldPassword(),user.getPassword())){
            throw new SaTokenException("原密码错误");
        }
        user.setPassword(BCrypt.hashpw(updatePswParam.getNewPassword()));
        return userMapper.updateById(user) > 0;
    }

}
